#! /usr/bin/python3
# Launcher script for the docker image of the ces slurm simulator
#
# This script is meant to ease the launching of the simulator. 
# it encapsulates the bulky docker commands and handles internal
# logic
#
# BSC-CNS - Earth Sciences
# Manuel Giménez de Castro Marciani
# 2023

import argparse
import os
import subprocess
import sys
from typing import List

# type hinting
Text = List[str]

def check_file(file_path:str) -> str:
    '''
    This function checks if the path exists, if not it tries to find the path 
    relative to the current directory.
    :param file_path: path to the file tested
    :returns: string with the updated path, incase it was relative
    '''

    abs_path = os.path.abspath(file_path)

    if not os.path.exists(abs_path):
        sys.exit(f'launcher.py: file {file_path} does not exist')

    return abs_path

def grep_image_tag(output_lines: Text, image:str, tag:str):
    '''
    searches through the lines for the image and tag. If it finds, returns true. False, otherwise.
    :param output_lines: list with the decoded utf-8, output of the command
    :param image: name of the docker image
    :param tag: tag of the docker image
    '''
    for line in output_lines:
        if image in line and tag in line:
            return True
    return False

def arg_parse():
    '''
    Parse arguments from the command line and handle dependencies between them
    :returns: argparse object with the updated 
    '''
    parser = argparse.ArgumentParser(description='Helper to launch the docker image')
    parser.add_argument('swf_path', type=str, help='relative or absolute path to the swf file')
    parser.add_argument('out_path', type=str, help='relative or absolute path to the output trace file')
    parser.add_argument('total_seconds', type=int, help='total number of simulated seconds')
    parser.add_argument('--db-dump-path', type=str, help='relative or absolute path to the sql dump path')
    parser.add_argument('--users-file-path', type=str, help='relative or absolute path to the users file')
    parser.add_argument('--slurm-template-path', type=str, help='relative or absolute path to the slurm template config file')
    parser.add_argument('--slurmctld-log', type=str, help='relative or absolute path to store the slurmctld log')
    parser.add_argument('--slurmd-log', type=str, help='relative or absolute path to store the slurmd log')
    parser.add_argument('--slurmdbd-log', type=str, help='relative or absolute path to store the slurmdbd log')
    parser.add_argument('--image', default='mgimenez/docker-ubuntu-ces-slurm-sim', type=str, help='select a different docker image to run')
    parser.add_argument('--tag', default='latest', type=str, help='select a different tag to run')

    arguments = parser.parse_args()

    arguments.swf_path = check_file(arguments.swf_path)
    arguments.out_path = check_file(arguments.out_path)

    if arguments.db_dump_path:
        arguments.db_dump_path = check_file(arguments.db_dump_path)
        if arguments.users_file_path:
            arguments.users_file_path = check_file(arguments.users_file_path)
        else:
            sys.exit('launcher.py: if you provide a database file, you also need to provide a user one')
    
    if arguments.slurm_template_path:
        arguments.slurm_template_path = check_file(arguments.slurm_template_path)

    if arguments.slurmctld_log:
        arguments.slurmctld_log = check_file(arguments.slurmctld_log)

    if arguments.slurmdbd_log:
        arguments.slurmdbd_log = check_file(arguments.slurmdbd_log)

    if arguments.slurmd_log:
        arguments.slurmd_log = check_file(arguments.slurmd_log)


    return arguments

if __name__ == '__main__':
    args = arg_parse()

    # here we check if the docker image exists, otherwise we build it :)
    docker_check = subprocess.run(['docker', 'images'], capture_output=True)

    if not grep_image_tag(docker_check.stdout.decode('utf-8').split('\n'), args.image, args.tag):
        sys.exit(f'launcher.py: {args.image}:{args.tag} does not exist.')

    # common to all cases
    command_to_exec = ['docker', 'run', '--rm', '-h', 'localhost',
                    '--mount', f'type=bind,source={args.swf_path},target=/opt/ces_slurm_simulator_tools/workloads/workload.swf',
                    '--mount', f'type=bind,source={args.out_path},target=/opt/ces_slurm_simulator_tools/simulation/TRACES/out.trace'] 
    end_line = [f'{args.image}:{args.tag}', '--end-time', f'{args.total_seconds}']

    if args.db_dump_path:
        command_to_exec += ['--mount', f'type=bind,source={args.users_file_path},target=/opt/ces_slurm_simulator_tools/install/slurm_conf/users.sim',
                            '--mount', f'type=bind,source={args.db_dump_path},target=/opt/ces_slurm_simulator_tools/slurm_sacct_db.sql']
        end_line += ["--not-create-users"]

    if args.slurm_template_path:
        command_to_exec += ['--mount', f'type=bind,source={args.slurm_template_path},target=/opt/ces_slurm_simulator_tools/install/slurm_conf/slurm.conf.template']
    
    if args.slurmctld_log:
        command_to_exec += ['--mount', f'type=bind,source={args.slurmctld_log},target=/opt/ces_slurm_simulator_tools/simulation/slurm_varios/log/slurmctld.log']

    if args.slurmd_log:
        command_to_exec += ['--mount', f'type=bind,source={args.slurmd_log},target=/opt/ces_slurm_simulator_tools/simulation/slurm_varios/log/slurmd.log']

    if args.slurmdbd_log:
        command_to_exec += ['--mount', f'type=bind,source={args.slurmdbd_log},target=/opt/ces_slurm_simulator_tools/install/slurm_varios/log/slurmdbd.log']

    docker_run = subprocess.run(command_to_exec+end_line)
